#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:source) do
  include EasyType
  include Puppet_X::EnterpriseModules::Oracle::Access

  desc <<-EOD
  A file describing the content of the procedure.

  Because puppet uses the contents of this file to decide if something needs to be done, the
  contents of this file must be very strict to the way Oracle stores the procedure source in the database.

  To decide if a change must be done, Puppet will fetch the procedure definition from the database and the contents
  of the source. For both it will remove all:

  - comments
  - empty lines
  - spaces
  - newlines
  - quotes

  and do a comparison.

  EOD

  to_translate_to_resource do |_raw_resource|
    'content-in-database'
  end

  #
  # Here we have the content and do the compare.
  #
  def insync?(_is)
    if current_content == required_content
      Puppet.warning "#{path}: procedure #{resource.owner}.#{resource.procedure_name} up-to-date, but contains #{compilation_errors} error(s)." \
        if contains_errors? && resource.report_errors
      true
    else
      current_content_file = Tempfile.new('ora_procedure')
      current_content_file.write(current_content)
      current_content_file.close
      required_content_file = Tempfile.new('ora_procedure')
      required_content_file.write(required_content)
      required_content_file.close
      file_diff = `diff #{current_content_file.path} #{required_content_file.path}`
      Puppet.info "#{path}: Difference is:\n#{file_diff}"
      false
    end
  end

  def change_to_s(currentvalue, _newvalue)
    if currentvalue == :absent
      'created procedure definition'
    else
      'changed to new procedure definition'
    end
  end

  def current_content
    @current_content ||= process_content(sql(template('ora_config/ora_procedure/procedure_definition.sql.erb', binding), :sid => resource.sid, :parse => false))
  end

  def required_content
    @required_content ||= process_content(File.read(value))
  end

  def process_content(content)
    new_content = content.gsub(/CONTENT\s*\n-*\s*\n/, '') \
                         .gsub(/--.*\n/, '') \
                         .gsub(%r{^/\s*$}, '') \
                         .gsub(/\s|\s|"|'/, '') \
                         .delete("\n").upcase
    if resource[:editionable] == :false
      new_content.gsub(/(?:non)?editionable/i, '')
    else
      new_content
    end
  end

  def compilation_errors
    results = sql template('ora_config/ora_procedure/procedure_errors.sql.erb', binding), :sid => resource.sid
    results.first['ERRORS'].to_i
  end

  def contains_errors?
    compilation_errors > 0
  end
end
